# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import logging

from .logger import setLevel, debug, info, warning, error
from .constants import DEBUG, INFO, WARNING, ERROR, CRITICAL, LOGGER_NAME

__all__ = [
    'setLevel', 'debug', 'info', 'warning', 'error', 'DEBUG', 'INFO',
    'WARNING', 'ERROR', 'CRITICAL', 'LOGGER_NAME'
]

logger = logging.getLogger(LOGGER_NAME)
logger.setLevel(logging.NOTSET)
stdout_handler = logging.StreamHandler()
stdout_handler.setFormatter(
    logging.Formatter('%(asctime)s [%(levelname)s] %(message)s'))
logger.addHandler(stdout_handler)



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class ESV(object):
    """
    ESV represents a ESV code in a protocol message of Echonet Lite.
    """
    UNKNOWN = 0x00
    WRITE_REQUEST = 0x60
    WRITE_REQUEST_RESPONSE_REQUIRED = 0x61
    READ_REQUEST = 0x62
    NOTIFICATION_REQUEST = 0x63
    WRITE_READ_REQUEST = 0x6E
    WRITE_RESPONSE = 0x71
    READ_RESPONSE = 0x72
    NOTIFICATION = 0x73
    NOTIFICATION_RESPONSE_REQUIRED = 0x74
    NOTIFICATION_RESPONSE = 0x7A
    WRITE_READ_RESPONSE = 0x7E
    WRITE_REQUEST_ERROR = 0x50
    WRITE_REQUEST_RESPONSE_REQUIRED_ERROR = 0x51
    READ_REQUEST_ERROR = 0x52
    NOTIFICATION_REQUEST_ERROR = 0x53
    WRITE_READ_REQUEST_ERROR = 0x5E

    def __init__(self, code=UNKNOWN):
        self.ESV = code



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .protocol.esv import ESV as ProtocolESV


class ESV(ProtocolESV):
    """
    ESV represents a ESV code in a protocol message of Echonet Lite.
    """
    def __init__(self):
        super(ESV, self).__init__()



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .object import Object


class Node(object):
    PORT = 3610

    def __init__(self):
        self.__address = ()
        self.__objects = {}

    def set_address(self, addr):
        if not isinstance(addr, tuple) or len(addr) != 2:
            return False
        self.__address = addr
        return True

    @property
    def address(self):
        return self.__address

    @property
    def ip(self):
        return self.__address[0]

    @property
    def port(self):
        return Node.PORT

    def add_object(self, obj):
        if not isinstance(obj, Object):
            return False
        self.__objects[obj.code] = obj
        return True

    @property
    def objects(self):
        objs = []
        for obj in self.__objects.values():
            objs.append(obj)
        return objs

    def has_object(self, code):
        return code in self.__objects.keys()



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .protocol.message import Message as ProtocolMessage
from .util import Bytes
from .node import Node
from .object import Object
from .node_profile import NodeProfile


class RemoteNode(Node):
    def __init__(self):
        super(RemoteNode, self).__init__()
        self.controller = None

    def parse_message(self, msg):
        if not isinstance(msg, ProtocolMessage):
            return False

        if msg.OPC < 1:
            return False

        prop = msg.properties[0]

        if prop.code != NodeProfile.CLASS_INSTANCE_LIST_NOTIFICATION and prop.code != NodeProfile.CLASS_SELF_NODE_INSTANCE_LIST_S:
            return False

        instance_count = prop.data[0]
        if len(prop.data) < ((instance_count * Object.CODE_SIZE) + 1):
            return False

        for n in range(instance_count):
            offset = (Object.CODE_SIZE * n) + 1
            code_bytes = prop.data[offset:(offset + Object.CODE_SIZE)]
            obj = Object()
            obj.set_code(Bytes.to_int(code_bytes))
            self.add_object(obj)

        return True



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..util.bytes import Bytes
from .esv import ESV
from .property import Property


class Message(ESV):
    """
    Message represents a protocol message of Echonet Lite.
    """
    FRAME_HEADER_SIZE = (1 + 1 + 2)
    FORMAT1_HEADER_SIZE = (3 + 3 + 1 + 1)
    FORMAT1_MIN_SIZE = (FRAME_HEADER_SIZE + FORMAT1_HEADER_SIZE)
    FORMAT1_PROPERTY_HEADER_SIZE = 2
    EHD1_ECHONET = 0x10
    EHD2_FORMAT1 = 0x81
    TID_SIZE = 2
    TID_MAX = 65535
    EOJ_SIZE = 3

    class ParserError(Exception):
        pass

    def __init__(self):
        super(Message, self).__init__()
        self.TID = 0
        self.SEOJ = 0
        self.DEOJ = 0
        self.properties = []
        self.from_addr = None
        self.to_addr = None

    @property
    def OPC(self):
        return len(self.properties)

    def is_response(self, msg):
        if self.TID != msg.TID:
            return False
        return True

    def parse_bytes(self, msg_bytes):
        # Frame heade
        if len(msg_bytes) < Message.FORMAT1_HEADER_SIZE:
            raise Message.ParserError()
        if msg_bytes[0] != Message.EHD1_ECHONET:
            raise Message.ParserError()
        if msg_bytes[1] != Message.EHD2_FORMAT1:
            raise Message.ParserError()
        self.TID = Bytes.to_int(msg_bytes[2:4])

        # Echonet Format1 Header
        self.SEOJ = Bytes.to_int(msg_bytes[4:7])
        self.DEOJ = Bytes.to_int(msg_bytes[7:10])
        self.ESV = msg_bytes[10]

        # Propety data
        opc = msg_bytes[11]
        offset = 12
        for n in range(opc):
            if len(msg_bytes) < (offset + 1):
                raise Message.ParserError()
            prop = Property()
            prop.code = msg_bytes[offset]
            offset += 1
            pdc = msg_bytes[offset]
            offset += 1
            if len(msg_bytes) < (offset + pdc):
                raise Message.ParserError()
            prop.data = msg_bytes[offset:(offset + pdc)]
            self.properties.append(prop)
            offset += pdc

        return True

    def to_bytes(self):
        msg_bytes = bytearray([Message.EHD1_ECHONET, Message.EHD2_FORMAT1])
        msg_bytes.extend(Bytes.from_int(self.TID, 2))
        msg_bytes.extend(Bytes.from_int(self.SEOJ, 3))
        msg_bytes.extend(Bytes.from_int(self.DEOJ, 3))
        msg_bytes.append(self.ESV)

        msg_bytes.append(len(self.properties))
        for prop in self.properties:
            msg_bytes.append(prop.code)
            msg_bytes.append(len(prop.data))
            msg_bytes.extend(prop.data)

        return msg_bytes

    def to_string(self):
        return self.to_bytes().hex().upper()



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time

from .log.logger import debug
from .transport.observer import Observer

from .local_node import LocalNode
from .node_profile import NodeProfile
from .esv import ESV
from .message import Message
from .property import Property
from .remote_node import RemoteNode
from .node import Node


class SearchMessage(Message):
    def __init__(self):
        super(SearchMessage, self).__init__()
        self.ESV = ESV.READ_REQUEST
        self.SEOJ = NodeProfile.OBJECT
        self.DEOJ = NodeProfile.OBJECT
        prop = Property()
        prop.code = NodeProfile.CLASS_SELF_NODE_INSTANCE_LIST_S
        prop.data = bytearray()
        self.add_property(prop)


class Controller(Observer):
    def __init__(self):
        self.node = LocalNode()
        self.found_nodes = {}
        self.__last_post_msg = Controller.__PostMessage()

    class __PostMessage():
        def __init__(self):
            self.request = None
            self.response = None

        def is_waiting(self):
            if self.request is None:
                return False
            if self.response is not None:
                return False
            return True

    @property
    def nodes(self):
        nodes = []
        for node in self.found_nodes.values():
            nodes.append(node)
        return nodes

    def __is_node_profile_message(self, msg):
        if msg.ESV != ESV.NOTIFICATION and msg.ESV != ESV.READ_RESPONSE:
            return False
        if msg.DEOJ != NodeProfile.OBJECT and msg.DEOJ != NodeProfile.OBJECT_READ_ONLY:
            return False
        return True

    def __add_found_node(self, node):
        if not isinstance(node, RemoteNode):
            return False
        node.controller = self
        self.found_nodes[node.ip] = node
        return True

    def message_received(self, msg):
        debug('%s -> %s' % (msg.from_addr[0].ljust(15), msg.to_string()))

        if self.__is_node_profile_message(msg):
            node = RemoteNode()
            node.set_address(msg.from_addr)
            if node.parse_message(msg):
                self.__add_found_node(node)

        if self.__last_post_msg.is_waiting():
            if self.__last_post_msg.request.is_response(msg):
                self.__last_post_msg.response = msg

    def announce_message(self, msg):
        return self.node.announce_message(msg)

    def send_message(self, msg, addr):
        to_addr = addr
        if isinstance(addr, RemoteNode):
            to_addr = (addr.ip, addr.port)
        elif isinstance(addr, str):
            to_addr = (addr, Node.PORT)
        return self.node.send_message(msg, to_addr)

    def search(self):
        msg = SearchMessage()
        return self.announce_message(msg)

    def post_message(self, msg, addr):
        self.__last_post_msg = Controller.__PostMessage()
        self.__last_post_msg.request = msg

        if not self.send_message(msg, addr):
            return None

        for i in range(10):
            time.sleep(0.2)
            if self.__last_post_msg.response is not None:
                break

        return self.__last_post_msg.response

    def start(self):
        if not self.node.start():
            return False
        self.node.add_observer(self)
        self.search()
        return True

    def stop(self):
        if not self.node.stop():
            return False
        return True



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .server import Server
from .multicast_server import MulticastServer


class UnicastServer(Server):
    def __init__(self):
        super(UnicastServer, self).__init__()

    def bind(self, ifaddr):
        self.socket = self.create_udp_socket()
        self.socket.bind((ifaddr, self.port))
        return True

    def announce_message(self, msg):
        if self.socket is None:
            return False
        to_addr = (MulticastServer.ADDRESS, Server.PORT)
        msg.to_addr = to_addr
        return self.socket.sendto(msg.to_bytes(), to_addr)

    def send_message(self, msg, addr):
        if not isinstance(addr, tuple) or len(addr) != 2:
            return False
        if self.socket is None:
            return False
        msg.to_addr = addr
        return self.socket.sendto(msg.to_bytes(), addr)



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License")
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .object import Object


class NodeProfile(Object):
    OBJECT = 0x0EF001
    OBJECT_READ_ONLY = 0x0EF002
    CLASS_CODE = 0xF0
    INSTANCE_GENERAL_CODE = 0x01
    INSTANCE_TRANSMISSION_ONLY_CODE = 0x02

    CLASS_OPERATING_STATUS = Object.OPERATING_STATUS
    CLASS_VERSION_INFORMATION = 0x82
    CLASS_IDENTIFICATION_NUMBER = 0x83
    CLASS_FAULT_CONTENT = 0x89
    CLASS_UNIQUE_IDENTIFIER_DATA = 0xBF
    CLASS_NUMBER_OF_SELF_NODE_INSTANCES = 0xD3
    CLASS_NUMBER_OF_SELF_NODE_CLASSES = 0xD4
    CLASS_INSTANCE_LIST_NOTIFICATION = 0xD5
    CLASS_SELF_NODE_INSTANCE_LIST_S = 0xD6
    CLASS_SELF_NODE_CLASS_LIST_S = 0xD7

    CLASS_OPERATING_STATUS_SIZE = 1
    CLASS_VERSION_INFORMATION_SIZE = 4
    CLASS_IDENTIFICATION_MANUFACTURER_CODE_SIZE = 3
    CLASS_IDENTIFICATION_UNIQUE_ID_SIZE = 13
    CLASS_IDENTIFICATION_NUMBER_SIZE = 1 + CLASS_IDENTIFICATION_MANUFACTURER_CODE_SIZE + CLASS_IDENTIFICATION_UNIQUE_ID_SIZE
    CLASS_FAULT_CONTENT_SIZE = 2
    CLASS_UNIQUE_IDENTIFIER_DATA_SIZE = 2
    CLASS_NUMBER_OF_SELF_NODE_INSTANCES_SIZE = 3
    CLASS_NUMBER_OF_SELF_NODE_CLASSES_SIZE = 2
    CLASS_SELF_NODE_INSTANCE_LIST_S_MAX = 0xFF
    CLASS_SELF_NODE_CLASS_LIST_S_MAX = 0xFF
    CLASS_INSTANCE_LIST_NOTIFICATION_MAX = CLASS_SELF_NODE_INSTANCE_LIST_S_MAX

    CLASS_OPERATING_STATUS_ON = Object.OPERATING_STATUS_ON
    CLASS_OPERATING_STATUS_OFF = Object.OPERATING_STATUS_OFF
    CLASS_BOOTING = 0x30
    CLASS_NOT_BOOTING = 0x31
    LOWER_COMMUNICATION_LAYER_PROTOCOL_TYPE = 0xFE

    def __init__(self):
        pass



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# from typing import List

from .interface import Interface
from .multicast_server import MulticastServer
from .unicast_server import UnicastServer
from ..log.logger import debug


class Manager(object):
    # servers: List[Server]

    def __init__(self):
        self.servers = []
        self.__TID = 0

    def __next_TID(self):
        self.__TID += 1
        if 0xFF < self.__TID:
            self.__TID = 0
        return self.__TID

    def add_observer(self, observer):
        for server in self.servers:
            server.add_observer(observer)

    def notify(self, msg):
        for server in self.servers:
            server.notify(msg)

    def announce_message(self, msg):
        msg.TID = self.__next_TID()
        debug('%s <- %s' %
              (MulticastServer.ADDRESS.ljust(15), msg.to_string()))
        for server in self.servers:
            if isinstance(server, UnicastServer):
                server.announce_message(msg)
        return True

    def send_message(self, msg, addr):
        msg.TID = self.__next_TID()
        debug('%s <- %s' % (addr[0].ljust(15), msg.to_string()))
        for server in self.servers:
            # TODO: Select an appropriate server from the specified address
            if isinstance(server, UnicastServer):
                if server.send_message(msg, addr):
                    return True
        return False

    def start(self):
        for ifaddr in Interface.get_all_ipaddrs():
            userver = UnicastServer()
            if not userver.bind(ifaddr):
                self.stop()
                return False
            if not userver.start():
                self.stop()
                return False
            self.servers.append(userver)

            mserver = MulticastServer()
            if not mserver.bind(ifaddr):
                self.stop()
                return False
            if not mserver.start():
                self.stop()
                return False
            self.servers.append(mserver)

        return True

    def stop(self):
        for server in self.servers:
            server.stop()
        self.servers = []
        return True



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket
from .server import Server


class MulticastServer(Server):
    ADDRESS = '224.0.23.0'

    def __init__(self):
        super(MulticastServer, self).__init__()

    def bind(self, ifaddr):
        self.socket = self.create_udp_socket()
        self.socket.bind(('0.0.0.0', self.port))
        opt = socket.inet_aton(
            MulticastServer.ADDRESS) + socket.inet_aton(ifaddr)
        self.socket.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP,
                               opt)
        return True



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License")
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .property import Property


class Object(object):
    CODE_MIN = 0x000000
    CODE_MAX = 0xFFFFFF
    CODE_SIZE = 3
    CODE_UNKNOWN = CODE_MIN

    OPERATING_STATUS = 0x80
    MANUFACTURER_CODE = 0x8A
    ANNO_PROPERTY_MAP = 0x9D
    SET_PROPERTY_MAP = 0x9E
    GET_PROPERTY_MAP = 0x9F

    OPERATING_STATUS_ON = 0x30
    OPERATING_STATUS_OFF = 0x31
    OPERATING_STATUS_SIZE = 1
    MANUFACTURER_EVALUATION_CODE_MIN = 0xFFFFF0
    MANUFACTURER_EVALUATION_CODE_MAX = 0xFFFFFF
    MANUFACTURER_CODE_SIZE = 3
    PROPERTY_MAP_MAX_SIZE = 16
    ANNO_PROPERTY_MAP_MAX_SIZE = PROPERTY_MAP_MAX_SIZE + 1
    SET_PROPERTY_MAP_MAX_SIZE = PROPERTY_MAP_MAX_SIZE + 1
    GET_PROPERTY_MAP_MAX_SIZE = PROPERTY_MAP_MAX_SIZE + 1

    MANUFACTURER_UNKNOWN = MANUFACTURER_EVALUATION_CODE_MIN

    def __init__(self):
        self.code = 0
        self.class_group_code = 0
        self.class_code = 0
        self.instance_code = 0
        self.properties = {}
        pass

    def set_code(self, code):
        if type(code) is int:
            self.code = code
            self.class_group_code = ((code >> 16) & 0xFF)
            self.class_code = ((code >> 8) & 0xFF)
            self.instance_code = (code & 0xFF)
            return True
        return False

    def add_property(self, prop):
        if not isinstance(prop, Property):
            return False
        self.properties[prop.code] = prop
        return True



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class Bytes():
    """
    Binary defines utility functions for converting between bytes and integer.
    """
    @classmethod
    def from_int(cls, val, bytes_size):
        byte_buf = bytearray(bytes_size)
        for n in range(bytes_size):
            idx = (bytes_size - 1) - n
            byte_buf[idx] = ((val >> (n * 8)) & 0xFF)
        return byte_buf

    @classmethod
    def to_int(cls, byte_buf):
        val = 0
        bytes_size = len(byte_buf)
        for n in range(bytes_size):
            idx = (bytes_size - 1) - n
            val += (byte_buf[idx]) << (n * 8)
        return val



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .protocol.message import Message as ProtocolMessage

from .util.bytes import Bytes
from .property import Property
from .object import Object
from .node_profile import NodeProfile


class Message(ProtocolMessage):
    def __init__(self):
        super(Message, self).__init__()

    def add_property(self, prop):
        if not isinstance(prop, Property):
            return False
        self.properties.append(prop)
        return True

    def add_object_as_class_instance_list_property(self, obj):
        if not isinstance(obj, Object):
            return False
        prop = Property()
        prop.code = NodeProfile.CLASS_SELF_NODE_INSTANCE_LIST_S
        prop_data = bytearray([1])
        prop_data.extend(Bytes.from_int(obj.code, Object.CODE_SIZE))
        prop.data = prop_data
        self.properties.append(prop)
        return True



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import netifaces


class Interface(object):
    @staticmethod
    def get_all_ipaddrs():
        ipaddrs = []
        for ifname in netifaces.interfaces():
            ifaddrs = netifaces.ifaddresses(ifname)
            if netifaces.AF_INET in ifaddrs.keys():
                inets = ifaddrs[netifaces.AF_INET]
                for inet in inets:
                    if 'addr' in inet.keys():
                        ipaddr = inet['addr']
                        if ipaddr == '127.0.0.1':
                            continue
                        ipaddrs.append(ipaddr)
        return ipaddrs



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License")
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .protocol.property import Property as ProtocolProperty


class Property(ProtocolProperty):
    CODE_MIN = 0x80
    CODE_MAX = 0xFF

    FORMAT1_MAX_SIZE = 15
    FORMAT2_SIZE = 18
    FORMAT_MAX_SIZE = FORMAT2_SIZE

    ATTRIBUTE_NONE = 0x00
    ATTRIBUTE_READ = 0x01
    ATTRIBUTE_WRITE = 0x02
    ATTRIBUTE_ANNO = 0x10
    ATTRIBUTE_READ_WRITE = ATTRIBUTE_READ | ATTRIBUTE_WRITE
    ATTRIBUTE_READ_ANNO = ATTRIBUTE_READ | ATTRIBUTE_ANNO

    def __init__(self):
        super(Property, self).__init__()
        self.attr = Property.ATTRIBUTE_NONE



#!/usr/bin/env python
# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools

with open("README.md", "r") as f:
    long_description=f.read()

setuptools.setup(
    name='uecho',
    version='0.8.2',
    description="uEcho for Python is a portable development framework for ECHONET Lite developers.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Satoshi Konno',
    author_email='skonno@cybergarage.org',
    url='https://github.com/cybergarage/uecho-py.git',
    install_requires=[
        'netifaces',
    ],
    packages=setuptools.find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: System :: Networking',
        'License :: OSI Approved :: Apache Software License',
    ],
)



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket
import threading
# from typing import List

from ..protocol.message import Message
from ..log.logger import error


class Server(threading.Thread):
    PORT = 3610

    # socket: socket.socket
    # port: int
    # observers: List[Observer]

    def __init__(self):
        super(Server, self).__init__()
        self.socket = None
        self.port = Server.PORT
        self.observers = []

    def create_udp_socket(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
        return sock

    def bind(self, ifaddr):
        pass

    def run(self):
        while True:
            try:
                if self.socket is None:
                    break
                recv_msg_bytes, recv_from = self.socket.recvfrom(1024)
                msg = Message()
                if not msg.parse_bytes(recv_msg_bytes):
                    log_msg = '%s %s' % (recv_from, recv_msg_bytes.hex())
                    error(log_msg)
                    continue
                msg.from_addr = recv_from
                self.notify(msg)
            except:
                break

    def add_observer(self, observer):
        self.observers.append(observer)

    def notify(self, msg):
        for observer in self.observers:
            observer.message_received(msg)

    def start(self):
        if self.socket is None:
            return False
        super(Server, self).start()
        return True

    def stop(self):
        if self.socket is None:
            return False
        self.socket.close()
        self.socket = None
        return True



# Copyright (C) 2021 Satoshi Konno. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .transport.manager import Manager

from .node_profile import NodeProfile


class LocalNode(Manager):
    def __init__(self):
        super(LocalNode, self).__init__()

    def announce_message(self, msg):
        msg.DEOJ = NodeProfile.OBJECT
        return super(LocalNode, self).announce_message(msg)

    def send_message(self, msg, addr):
        msg.DEOJ = NodeProfile.OBJECT
        return super(LocalNode, self).send_message(msg, addr)
